@@ -1,4 +1,4 @@
-/* Copyright (c) 1999, 2000, 2001, 2005, 2006, 2010, 2011, 2012 Thorsten Kukuk
+/* Copyright (c) 1999, 2000, 2001, 2005, 2006, 2010, 2011, 2012, 2014, 2015, 2016 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -19,30 +19,26 @@
 #include "config.h"
 #endif
 
-#include <pwd.h>
 #include <time.h>
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
-#ifdef HAVE_ALLOCA_H
 #include <alloca.h>
-#endif /* HAVE_ALLOCA_H */
 #include <stdlib.h>
+#include <crypt.h>
+#include <shadow.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <rpc/rpc.h>
-#include <rpc/pmap_clnt.h>
-#include "yppasswd.h"
+#include <rpcsvc/yp_prot.h>
+#define passwd xpasswd
+#include <rpcsvc/yppasswd.h>
+#undef passwd
+#include <pwd.h>
+#include "yppwd_local.h"
 #include "log_msg.h"
-#ifdef HAVE_CRYPT_H
-#include <crypt.h>
-#endif
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif
-#include "compat.h"
 
 #ifndef CHECKROOT
 /* Set to 0 if you don't want to check against the root password
@@ -53,11 +49,9 @@
 #ifndef _PATH_PASSWD
 #define _PATH_PASSWD            "/etc/passwd"
 #endif
-#ifdef HAVE_GETSPNAM
 #ifndef _PATH_SHADOW
 #define _PATH_SHADOW            "/etc/shadow"
 #endif
-#endif
 #ifndef _PATH_SHELLS
 #define _PATH_SHELLS            "/etc/shells"
 #endif
@@ -68,11 +62,9 @@
 char *path_passwd = _PATH_PASSWD;
 char *path_passwd_tmp = NULL;
 char *path_passwd_old = NULL;
-#ifdef HAVE_GETSPNAM
 char *path_shadow = _PATH_SHADOW;
 char *path_shadow_tmp = NULL;
 char *path_shadow_old = NULL;
-#endif
 
 /* Will be set by the main function */
 char *external_update_program = NULL;
@@ -80,8 +72,8 @@ char *external_update_program = NULL;
 static bool_t adjuct_used = FALSE;
 
 static int external_update_env (yppasswd *yppw);
-static int external_update_pipe (yppasswd *yppw, char *logbuf);
-static int update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
+static int external_update_pipe (yppasswd *yppw);
+static int update_files (yppasswd *yppw, int *shadow_changed,
 			 int *passwd_changed, int *chfn, int *chsh);
 
 /* Argument validation. Avoid \n... (ouch).
@@ -214,7 +206,7 @@ putspent_adjunct (const struct spwd *p, FILE *stream)
 
 /* Check if the password the user supplied matches the old one */
 static int
-password_ok (char *plain, char *crypted, char *root, char *logbuf)
+password_ok (char *plain, char *crypted, char *root)
 {
   char *crypted_new;
   if (crypted[0] == '\0')
@@ -222,7 +214,7 @@ password_ok (char *plain, char *crypted, char *root, char *logbuf)
   crypted_new = crypt (plain, crypted);
   if (crypted_new == NULL)
     {
-      log_msg ("crypt() call failed.", logbuf);
+      log_msg ("crypt() call failed.");
       return 0;
     }
   if (strcmp (crypted_new, crypted) == 0)
@@ -231,7 +223,7 @@ password_ok (char *plain, char *crypted, char *root, char *logbuf)
   crypted_new = crypt (plain, root);
   if (crypted_new == NULL)
     {
-      log_msg ("crypt() call failed.", logbuf);
+      log_msg ("crypt() call failed.");
       return 0;
     }
   if (strcmp (crypted_new, root) == 0)
@@ -241,7 +233,6 @@ password_ok (char *plain, char *crypted, char *root, char *logbuf)
   return 0;
 }
 
-#ifdef HAVE_GETSPNAM
 static inline int
 is_allowed_to_change (const struct spwd *sp)
 {
@@ -262,7 +253,6 @@ is_allowed_to_change (const struct spwd *sp)
 
   return 1;
 }
-#endif
 
 /*********************************************************************
  * The Update Handler                                                *
@@ -271,20 +261,24 @@ is_allowed_to_change (const struct spwd *sp)
 int *
 yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 {
+  char namebuf6[INET6_ADDRSTRLEN];
   int shadow_changed = 0, passwd_changed = 0, chsh = 0, chfn = 0;
   int retries;
   static int res;                /* I hate static variables */
-  char *logbuf;
-  const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+  struct netconfig *nconf;
+  const struct netbuf *rqhost = svc_getrpccaller (rqstp->rq_xprt);
+
+  nconf = getnetconfigent (rqstp->rq_xprt->xp_netid);
 
   /* Be careful here with the debug option. You can see the old
      and new password in clear text !! */
   if (debug_flag)
     {
-      log_msg ("yppasswdproc_pwupdate(\"%s\") [From: %s:%d]",
-              yppw->newpw.pw_name,
-              inet_ntoa (rqhost->sin_addr),
-              ntohs (rqhost->sin_port));
+      log_msg ("yppasswdproc_pwupdate(\"%s\") [From: %s port %d]",
+	       yppw->newpw.pw_name,
+	       taddr2ipstr (nconf, rqhost,
+			    namebuf6, sizeof (namebuf6)),
+	       taddr2port (nconf, rqhost));
       log_msg ("\toldpass..: %s", yppw->oldpass);
       log_msg ("\tpw_name..: %s", yppw->newpw.pw_name);
       log_msg ("\tpw_passwd: %s", yppw->newpw.pw_passwd);
@@ -295,22 +289,14 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 
   res = 1; /* res = 1 means no success */
 
-  logbuf = alloca (60 + strlen (yppw->newpw.pw_name) +
-                   strlen (inet_ntoa (rqhost->sin_addr)));
-  if (logbuf == NULL)
-    {
-      log_msg ("rpc.yppasswdd: out of memory");
-      return &res;
-    }
-
-  sprintf (logbuf, "update %.12s (uid=%d) from host %s",
-           yppw->newpw.pw_name, yppw->newpw.pw_uid,
-	   inet_ntoa (rqhost->sin_addr));
-
   /* Check if somebody tries to make trouble with not allowed characters */
   if (!validate_args (&yppw->newpw))
     {
-      log_msg ("%s failed", logbuf);
+      log_msg ("update %.12s (uid=%d) from host %s failed",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid,
+	       taddr2ipstr (nconf, rqhost,
+			    namebuf6, sizeof (namebuf6)));
+      freenetconfigent (nconf);
       return &res;
     }
 
@@ -322,6 +308,7 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
       if ((pw = getpwnam (yppw->newpw.pw_name)) == NULL)
 	{
 	  log_msg ("user %s not found", yppw->newpw.pw_name);
+	  freenetconfigent (nconf);
 	  return &res;
 	}
       /* Do we need to update the GECOS information and are we allowed
@@ -329,8 +316,12 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
       chfn = (strcmp (pw->pw_gecos, yppw->newpw.pw_gecos) != 0);
       if (chfn && !allow_chfn)
 	{
-	  log_msg ("%s rejected", logbuf);
+	  log_msg ("update %.12s (uid=%d) from host %s rejected",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid,
+		   taddr2ipstr (nconf, rqhost,
+				namebuf6, sizeof (namebuf6)));
 	  log_msg ("chfn not permitted");
+	  freenetconfigent (nconf);
 	  return &res;
 	}
 
@@ -340,35 +331,46 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 	{
 	  if (!allow_chsh)
 	    {
-	      log_msg ("%s rejected", logbuf);
+	      log_msg ("update %.12s (uid=%d) from host %s rejected",
+		       yppw->newpw.pw_name, yppw->newpw.pw_uid,
+		       taddr2ipstr (nconf, rqhost,
+				    namebuf6, sizeof (namebuf6)));
 	      log_msg ("chsh not permitted");
+	      freenetconfigent (nconf);
 	      return &res;
 	    }
 	  if (!shell_ok (yppw->newpw.pw_shell))
 	    {
-	      log_msg ("%s rejected", logbuf);
+	      log_msg ("update %.12s (uid=%d) from host %s rejected",
+		       yppw->newpw.pw_name, yppw->newpw.pw_uid,
+		       taddr2ipstr (nconf, rqhost,
+				    namebuf6, sizeof (namebuf6)));
 	      log_msg ("invalid shell: %s", yppw->newpw.pw_shell);
+	      freenetconfigent (nconf);
 	      return &res;
 	    }
 	}
 
       if (x_flag)
         {
-          res = external_update_pipe (yppw, logbuf);
+          res = external_update_pipe (yppw);
+	  freenetconfigent (nconf);
           return &res;
         }
       else
         {
           res = external_update_env (yppw);
           if (res >= 2)
-            return &res;
+	    {
+	      freenetconfigent (nconf);
+	      return &res;
+	    }
         }
       passwd_changed = 1; /* We don't know exactly what was changed. */
       shadow_changed = 1; /* So build everything new. */
     }
   else
     {
-#ifdef HAVE_LCKPWDF
       /* Lock the passwd file. We retry several times. */
       retries = 0;
       while (lckpwdf () && retries < MAX_RETRIES)
@@ -379,18 +381,19 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 
       if (retries == MAX_RETRIES)
 	{
-	  log_msg ("%s failed", logbuf);
+	  log_msg ("update %.12s (uid=%d) from host %s failed",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid,
+		   taddr2ipstr (nconf, rqhost,
+				namebuf6, sizeof (namebuf6)));
 	  log_msg ("password file locked");
+	  freenetconfigent (nconf);
 	  return &res;
 	}
-#endif /* HAVE_LCKPWDF */
 
-      res = update_files (yppw, logbuf, &shadow_changed, &passwd_changed,
+      res = update_files (yppw, &shadow_changed, &passwd_changed,
 			  &chfn, &chsh);
 
-#ifdef HAVE_LCKPWDF
       ulckpwdf ();
-#endif /* HAVE_LCKPWDF */
     }
 
   /* Fork off process to rebuild NIS passwd.* maps. */
@@ -405,8 +408,12 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
         {
           /* Do NOT restore old password file. Someone else may already
            * be using the new one. */
-          log_msg ("%s failed", logbuf);
+	  log_msg ("update %.12s (uid=%d) from host %s failed",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid,
+		   taddr2ipstr (nconf, rqhost,
+				namebuf6, sizeof (namebuf6)));
           log_msg ("Couldn't fork map update process: %s", strerror (errno));
+	  freenetconfigent (nconf);
           return &res;
         }
 
@@ -421,15 +428,19 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
           exit (1);
         }
 
-      log_msg ("%s successful.", logbuf);
+      log_msg ("update %.12s (uid=%d) from host %s successful",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid,
+	       taddr2ipstr (nconf, rqhost,
+			    namebuf6, sizeof (namebuf6)));
       if (chsh || chfn)
 	{
 	  log_msg ("Shell %schanged (%s), GECOS %schanged (%s).",
-		  chsh ? "" : "un", yppw->newpw.pw_shell,
-		  chfn ? "" : "un", yppw->newpw.pw_gecos);
+		   chsh ? "" : "un", yppw->newpw.pw_shell,
+		   chfn ? "" : "un", yppw->newpw.pw_gecos);
 	}
     }
 
+  freenetconfigent (nconf);
   return &res;
 }
 
@@ -439,7 +450,7 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
   1: error
 */
 static int
-update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
+update_files (yppasswd *yppw, int *shadow_changed,
 	      int *passwd_changed, int *chfn, int *chsh)
 {
   struct passwd *pw;
@@ -454,7 +465,6 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
     {
       if (strcmp (pw->pw_passwd, "x") == 0)
 	{
-#ifdef HAVE_GETSPNAM /* shadow password */
 	  struct spwd *spw;
 
 	  if ((spw = getspnam ("root")) != NULL)
@@ -462,7 +472,6 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	      rootpass = alloca (strlen (spw->sp_pwdp) + 1);
 	      strcpy (rootpass, spw->sp_pwdp);
 	    }
-#endif /* HAVE_GETSPNAM */
 	}
       else
 	{
@@ -476,14 +485,16 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
      friends here. */
   if ((oldpf = fopen (path_passwd, "r")) == NULL)
     {
-      log_msg ("%s failed", logbuf);
+      log_msg ("update %.12s (uid=%d) failed",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       log_msg ("Can't open %s: %m", path_passwd);
       return 1;
     }
 
   if (fstat (fileno (oldpf), &passwd_stat) < 0)
     {
-      log_msg ("%s failed", logbuf);
+      log_msg ("update %.12s (uid=%d) failed",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       log_msg ("Can't stat %s: %m", path_passwd);
       fclose (oldpf);
       return 1;
@@ -492,7 +503,8 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   /* Open a temp passwd file */
   if ((newpf = fopen (path_passwd_tmp, "w+")) == NULL)
     {
-      log_msg ("%s failed", logbuf);
+      log_msg ("update %.12s (uid=%d) failed",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       log_msg ("Can't open %s: %m", path_passwd_tmp);
       fclose (oldpf);
       return 1;
@@ -507,13 +519,13 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       return 1;
     }
 
-#ifdef HAVE_GETSPNAM
   /* Open the shadow file for reading. */
   if ((oldsf = fopen (path_shadow, "r")) != NULL)
     {
       if (fstat (fileno (oldsf), &shadow_stat) < 0)
 	{
-	  log_msg ("%s failed", logbuf);
+	  log_msg ("update %.12s (uid=%d) failed",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 	  log_msg ("Can't stat %s: %m", path_shadow);
 	  fclose (oldpf);
 	  fclose (newpf);
@@ -524,7 +536,9 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       if ((newsf = fopen (path_shadow_tmp, "w+")) == NULL)
 	{
 	  int err = errno;
-	  log_msg ("%s failed", logbuf);
+
+	  log_msg ("update %.12s (uid=%d) failed",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 	  log_msg ("Can't open %s.tmp: %s",
 		   path_passwd, strerror (err));
 	  fclose (oldsf);
@@ -544,7 +558,6 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	  return 1;
 	}
     }
-#endif /* HAVE_GETSPNAM */
 
   /* Loop over all passwd entries */
   while ((pw = fgetpwent (oldpf)) != NULL)
@@ -562,15 +575,15 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	      ((pw->pw_passwd[0] == 'x' && pw->pw_passwd[1] == '\0') ||
               (pw->pw_passwd[0] == '#' && pw->pw_passwd[1] == '#')))
 	    {
-#ifdef HAVE_GETSPNAM /* shadow password */
 	      /* Search for the shadow entry of this user */
 	      while ((spw = fgetspent_adjunct (oldsf)) != NULL)
 		{
 		  if (strcmp (yppw->newpw.pw_name, spw->sp_namp) == 0)
 		    {
-		      if (!password_ok (yppw->oldpass, spw->sp_pwdp, rootpass, logbuf))
+		      if (!password_ok (yppw->oldpass, spw->sp_pwdp, rootpass))
 			{
-			  log_msg ("%s rejected", logbuf);
+			  log_msg ("update %.12s (uid=%d) failed",
+				   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 			  log_msg ("Invalid password.");
 			  goto error;
 			}
@@ -579,20 +592,21 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		    }
 		  else if (putspent_adjunct (spw, newsf) < 0)
 		    {
-		      log_msg ("%s failed", logbuf);
+		      log_msg ("update %.12s (uid=%d) failed",
+			       yppw->newpw.pw_name, yppw->newpw.pw_uid);
 		      log_msg ("Error while writing new shadow file: %m");
 		      goto error;
 		    }
 		}
-#endif /* HAVE_GETSPNAM */
 	    }
 
 	  /* We don't have a shadow password file or we don't find the
 	     user in it. */
 	  if (spw == NULL &&
-	      !password_ok (yppw->oldpass, pw->pw_passwd, rootpass, logbuf))
+	      !password_ok (yppw->oldpass, pw->pw_passwd, rootpass))
 	    {
-	      log_msg ("%s rejected", logbuf);
+	      log_msg ("update %.12s (uid=%d) rekected",
+		       yppw->newpw.pw_name, yppw->newpw.pw_uid);
 	      log_msg ("Invalid password.");
 	      goto error;
 	    }
@@ -605,7 +619,6 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		yppw->newpw.pw_passwd[1] == '\0') &&
 	      yppw->newpw.pw_passwd[0] != '\0')
 	    {
-#ifdef HAVE_GETSPNAM /* shadow password */
 	      if (spw)
 		{
 		  /* test if password is expired */
@@ -623,7 +636,8 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 			}
 		      else
 			{
-			  log_msg ("%s rejected", logbuf);
+			  log_msg ("update %.12s (uid=%d) rejected",
+				   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 			  log_msg ("now < minimum age for `%s'",
 				  spw->sp_namp);
 			  goto error;
@@ -631,7 +645,8 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		    }
 		  if (putspent_adjunct (spw, newsf) < 0)
 		    {
-		      log_msg ("%s failed", logbuf);
+		      log_msg ("update %.12s (uid=%d) failed",
+			       yppw->newpw.pw_name, yppw->newpw.pw_uid);
 		      log_msg ("Error while writing new shadow file: %m");
 		      *shadow_changed = 0;
 		      goto error;
@@ -641,14 +656,14 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		  while ((spw = fgetspent_adjunct (oldsf)) != NULL)
 		    if (putspent_adjunct (spw, newsf) < 0)
 		      {
-			log_msg ("%s failed", logbuf);
+			log_msg ("update %.12s (uid=%d) failed",
+				 yppw->newpw.pw_name, yppw->newpw.pw_uid);
 			log_msg ("Error while writing new shadow file: %m");
 			*shadow_changed = 0;
 			goto error;
 		      }
 		}
 	      else /* No shadow entry */
-#endif /* HAVE_GETSPNAM */
 		{
 		  /* set the new passwd */
 		  pw->pw_passwd = yppw->newpw.pw_passwd;
@@ -666,7 +681,8 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	    {
 	      if (!allow_chfn)
 		{
-		  log_msg ("%s rejected", logbuf);
+		  log_msg ("update %.12s (uid=%d) rejected",
+			   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 		  log_msg ("chfn not permitted");
 		  *passwd_changed = 0;
 		  goto error;
@@ -681,14 +697,16 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	    {
 	      if (!allow_chsh)
 		{
-		  log_msg ("%s rejected", logbuf);
+		  log_msg ("update %.12s (uid=%d) rejected",
+			   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 		  log_msg ("chsh not permitted");
 		  *passwd_changed = 0;
 		  goto error;
 		}
 	      if (!shell_ok (yppw->newpw.pw_shell))
 		{
-		  log_msg ("%s rejected", logbuf);
+		  log_msg ("update %.12s (uid=%d) rejected",
+			   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 		  log_msg ("invalid shell: %s", yppw->newpw.pw_shell);
 		  *passwd_changed = 0;
 		  goto error;
@@ -702,7 +720,9 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       if (putpwent (pw, newpf) < 0)
 	{
 	  int err = errno;
-	  log_msg ("%s failed", logbuf);
+
+	  log_msg ("update %.12s (uid=%d) failed",
+		   yppw->newpw.pw_name, yppw->newpw.pw_uid);
 	  log_msg ("Error while writing new password file: %s",
 		   strerror (err));
 	  *passwd_changed = 0;
@@ -719,12 +739,9 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   if (pw || spw)
     {
       unlink (path_passwd_tmp);
-#ifdef HAVE_GETSPNAM
       unlink (path_shadow_tmp);
-#endif /* HAVE_GETSPNAM */
       return 1;
     }
-#ifdef HAVE_GETSPNAM
   if (*shadow_changed)
     {
       unlink (path_shadow_old);
@@ -740,7 +757,6 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
     }
   else
     unlink (path_shadow_tmp);
-#endif /* HAVE_GETSPNAM */
 
   if (*passwd_changed)
     {
@@ -778,15 +794,11 @@ external_update_env (yppasswd *yppw)
     }
   else
     { /* Child - run external update program */
-#if defined(HAVE_SETENV)
       setenv ("YP_PASSWD_OLD", yppw->oldpass, 1);
       setenv ("YP_PASSWD_NEW", yppw->newpw.pw_passwd, 1);
       setenv ("YP_USER", yppw->newpw.pw_name, 1);
       setenv ("YP_GECOS", yppw->newpw.pw_gecos, 1);
       setenv ("YP_SHELL", yppw->newpw.pw_shell, 1);
-#else
-#  error "Missing setenv(). Need porting."
-#endif
       execlp (external_update_program, external_update_program, NULL);
       _exit (1); /* fall-through */
     }
@@ -858,9 +870,9 @@ remove_password (char *str)
 }
 
 static int
-external_update_pipe (yppasswd *yppw, char *logbuf)
+external_update_pipe (yppasswd *yppw)
 {
-  struct xpasswd *newpw;       /* passwd struct passed by the client */
+  struct passwd *newpw;       /* passwd struct passed by the client */
   int res, pid, tochildpipe[2], toparentpipe[2];
   FILE *fp;
   char childresponse[1024];
@@ -874,7 +886,7 @@ external_update_pipe (yppasswd *yppw, char *logbuf)
 
   /* - */
 
-  newpw = &yppw->newpw;
+  newpw = (struct passwd *)&yppw->newpw;
   res = 1;
 
   /*
@@ -892,7 +904,8 @@ external_update_pipe (yppasswd *yppw, char *logbuf)
 
   if (!password && !shell && !gcos)
     {
-      log_msg ("%s failed - no information to change", logbuf);
+      log_msg ("update %.12s (uid=%d) failed - no information to change",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       return res;
     }
 
@@ -903,19 +916,22 @@ external_update_pipe (yppasswd *yppw, char *logbuf)
 
   if (pipe(tochildpipe) < 0)
     {
-      log_msg ("%s failed - couldn't create child pipe", logbuf);
+      log_msg ("update %.12s (uid=%d) failed - could not create child pipe",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       return res;
     }
 
   if (pipe(toparentpipe) < 0)
     {
-      log_msg ("%s failed - couldn't create parent pipe", logbuf);
+      log_msg ("update %.12s (uid=%d) failed - could not create parent pipe",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       return res;
     }
 
   if ((pid = fork()) < 0)
     {
-      log_msg ("%s failed - couldn't fork", logbuf);
+      log_msg ("update %.12s (uid=%d) failed - could not fork",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid);
       return res;
     }
 
@@ -1036,13 +1052,15 @@ external_update_pipe (yppasswd *yppw, char *logbuf)
 
   if (strspn(childresponse, "OK") < 2)
     {
-      log_msg ("%s failed.  Change request: %s", logbuf, parentmsg);
+      log_msg ("update %.12s (uid=%d) failed. Change request: %s",
+	       yppw->newpw.pw_name, yppw->newpw.pw_uid, parentmsg);
       log_msg ("Response was '%s'", childresponse);
       free (parentmsg);
       return res;
     }
 
-  log_msg ("%s successful. Change request: %s", logbuf, parentmsg);
+  log_msg ("update %.12s (uid=%d) successful. Change request: %s",
+	   yppw->newpw.pw_name, yppw->newpw.pw_uid, parentmsg);
 
   free (parentmsg);
   res = 0;
