@@ -1,4 +1,4 @@
-/* Copyright (c) 1996-1999, 2001-2003, 2005, 2006, 2010 Thorsten Kukuk
+/* Copyright (c) 1996-1999, 2001-2003, 2005, 2006, 2010, 2014, 2015 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -30,22 +30,14 @@
 #include <memory.h>
 #include <unistd.h>
 #include <syslog.h>
-#ifndef LOG_DAEMON
-#include <sys/syslog.h>
-#endif
 #include <signal.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <rpc/rpc.h>
-#ifdef HAVE_RPC_SVC_SOC_H
-#include <rpc/svc_soc.h>
-#endif /* HAVE_RPC_SVC_SOC_H */
-#include <rpc/pmap_clnt.h>
-#ifdef HAVE_GETOPT_H
+#include <rpc/nettype.h>
 #include <getopt.h>
-#endif /* HAVE_GETOPT_H */
 #include "ypxfrd.h"
 #include "access.h"
 #include "ypserv_conf.h"
@@ -55,12 +47,11 @@
 #endif
 
 #include "log_msg.h"
-#include "compat.h"
 #include "pidfile.h"
 
 #define _YPXFRD_PIDFILE _PATH_VARRUN"ypxfrd.pid"
 
-extern void ypxfrd_freebsd_prog_1(struct svc_req *, SVCXPRT *);
+extern void ypxfrd_freebsd_prog_1 (struct svc_req *, SVCXPRT *);
 
 int _rpcpmstart = 0;
 int _rpcfdtype = 0;
@@ -70,11 +61,7 @@ int _rpcsvcdirty = 0;
 #define _RPCSVC_CLOSEDOWN       120
 #endif
 
-#ifndef YPMAPDIR
-#define YPMAPDIR   "/var/yp"
-#endif
 char *path_ypdb = YPMAPDIR;
-
 char *progname;
 
 static int foreground_flag = 0;
@@ -94,7 +81,7 @@ closedown (int sig)
       if (_rpcfdtype == SOCK_DGRAM)
 	exit(0);
       if (size == 0)
-	size = _rpc_dtablesize();
+	size = svc_maxfd+1;
 
       for (i = 0, openfd = 0; i < size && openfd < 2; ++i)
 	if (FD_ISSET(i, &svc_fdset))
@@ -120,7 +107,7 @@ sig_child (int sig UNUSED)
 static void
 sig_quit (int sig UNUSED)
 {
-  pmap_unset (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS);
+  svc_unreg (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS);
   unlink (_YPXFRD_PIDFILE);
   exit (0);
 }
@@ -149,17 +136,10 @@ usage (int exitcode)
 int
 main (int argc, char **argv)
 {
-  SVCXPRT *main_transp;
+  struct netconfig *nconf;
+  void *nc_handle;
   int my_port = -1;
-  int my_socket;
-  struct sockaddr_in socket_address;
-  int result;
   struct sigaction sa;
-#if defined(__hpux)
-  int socket_size;
-#else /* not __hpux */
-  socklen_t socket_size;
-#endif
 
   progname = strrchr (argv[0], '/');
   if (progname == (char *) NULL)
@@ -206,7 +186,7 @@ main (int argc, char **argv)
 	  my_port = atoi(optarg);
 	  if (my_port <= 0 || my_port > 0xffff) {
 	    /* Invalid port number */
-	    fprintf (stdout, "Warning: rpc.ypxfrd: Invalid port %d (0x%x)\n", 
+	    fprintf (stdout, "Warning: rpc.ypxfrd: Invalid port %d (0x%x)\n",
 			my_port, my_port);
 	    my_port = -1;
 	  }
@@ -231,7 +211,7 @@ main (int argc, char **argv)
 
   if (debug_flag)
     log_msg("[Welcome to the rpc.ypxfrd Daemon, version %s]\n", VERSION);
-  else 
+  else
     if (!_rpcpmstart && !foreground_flag)
       {
 	int i;
@@ -350,105 +330,128 @@ main (int argc, char **argv)
   sigemptyset(&sa.sa_mask);
   sigaction(SIGHUP, &sa, NULL);
 
-  pmap_unset(YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS);
-
-  socket_size = sizeof(socket_address);
-  _rpcfdtype = 0;
-  if (getsockname(0, (struct sockaddr *)&socket_address, &socket_size) == 0)
+  rpcb_unset(YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS, NULL);
+  nc_handle = __rpc_setconf ("netpath");   /* open netconfig file */
+  if (nc_handle == NULL)
     {
-#if defined(__hpux)
-      int int_size = sizeof (int);
-#else /* not __hpux */
-      socklen_t  int_size = sizeof (int);
-#endif
-      if (socket_address.sin_family != AF_INET)
-	return 1;
-      if (getsockopt(0, SOL_SOCKET, SO_TYPE, (void*)&_rpcfdtype,
-		     &int_size) == -1)
-	return 1;
-      _rpcpmstart = 1;
-      my_socket = 0;
+      log_msg ("could not read /etc/netconfig, exiting..");
+      return 1;
     }
-  else
-    my_socket = RPC_ANYSOCK;
 
-  if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_DGRAM))
+  while ((nconf = __rpc_getconf (nc_handle)))
     {
-      if (_rpcfdtype == 0 && my_port > 0)
-	{
-	  my_socket = socket (AF_INET, SOCK_DGRAM, 0);
-	  if (my_socket < 0)
-	    {
-	      log_msg("can not create UDP: %s",strerror(errno));
-	      return 1;
-	    }
+      SVCXPRT *xprt;
+      struct sockaddr *sa;
+      struct sockaddr_in sin;
+      struct sockaddr_in6 sin6;
+      int sock;
+      sa_family_t family; /* AF_INET, AF_INET6 */
+      int type; /* SOCK_DGRAM (udp), SOCK_STREAM (tcp) */
+      int proto; /* IPPROTO_UDP, IPPROTO_TCP */
 
-	  memset((char *) &socket_address, 0, sizeof(socket_address));
-	  socket_address.sin_family = AF_INET;
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (my_port);
+      if (debug_flag)
+        log_msg ("Register ypserv for %s,%s",
+                 nconf->nc_protofmly, nconf->nc_proto);
 
-	  result = bind (my_socket, (struct sockaddr *) &socket_address,
-			 sizeof (socket_address));
-	  if (result < 0)
-	    {
-	      log_msg("%s: can not bind UDP: %s ", progname,strerror(errno));
-	      return 1;
-	    }
-	}
+      if (strcmp (nconf->nc_protofmly, "inet6") == 0)
+        family = AF_INET6;
+      else if (strcmp (nconf->nc_protofmly, "inet") == 0)
+        family = AF_INET;
+      else
+        continue; /* we don't support nconf->nc_protofmly */
 
-      main_transp = svcudp_create(my_socket);
-      if (main_transp == NULL)
+      if (strcmp (nconf->nc_proto, "udp") == 0)
+        {
+          type = SOCK_DGRAM;
+          proto = IPPROTO_UDP;
+        }
+      else if (strcmp (nconf->nc_proto, "tcp") == 0)
+        {
+          type = SOCK_STREAM;
+          proto = IPPROTO_TCP;
+        }
+      else
+        continue; /* We don't support nconf->nc_proto */
+
+      if ((sock = socket (family, type, proto)) < 0)
+        {
+          log_msg ("Cannot create socket for %s,%s: %s",
+                   nconf->nc_protofmly, nconf->nc_proto,
+                   strerror (errno));
+          continue;
+        }
+
+      if (family == AF_INET6)
+        {
+          /* Disallow v4-in-v6 to allow host-based access checks */
+
+          int i;
+
+          if (setsockopt (sock, IPPROTO_IPV6, IPV6_V6ONLY,
+                          &i, sizeof(i)) == -1)
+            {
+              log_msg ("ERROR: cannot disable v4-in-v6 on %s6 socket",
+                       nconf->nc_proto);
+              return 1;
+            }
+        }
+
+      switch (family)
+        {
+        case AF_INET:
+          memset (&sin, 0, sizeof(sin));
+          sin.sin_family = AF_INET;
+          if (my_port > 0)
+            sin.sin_port = htons (my_port);
+          sa = (struct sockaddr *)(void *)&sin;
+          break;
+        case AF_INET6:
+          memset (&sin6, 0, sizeof (sin6));
+          sin6.sin6_family = AF_INET6;
+          if (my_port > 0)
+            sin6.sin6_port = htons (my_port);
+          sa = (struct sockaddr *)(void *)&sin6;
+          break;
+        default:
+          log_msg ("Unsupported address family %d", family);
+          return -1;
+        }
+
+      if (bindresvport_sa (sock, sa) == -1)
+        {
+          if (my_port > 0)
+            log_msg ("Cannot bind to reserved port %d (%s)",
+                     my_port, strerror (errno));
+          else
+            log_msg ("bindresvport failed: %s",
+                     strerror (errno));
+          return 1;
+        }
+
+      if (type == SOCK_STREAM)
+        {
+          listen (sock, SOMAXCONN);
+          xprt = svc_vc_create (sock, 0, 0);
+        }
+      else
+        xprt = svc_dg_create (sock, 0, 0);
+
+      if (xprt == NULL)
+        {
+          log_msg ("terminating: cannot create rpcbind handle");
+          return 1;
+        }
+
+      rpcb_unset (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS, nconf);
+      if (!svc_reg (xprt, YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
+		    ypxfrd_freebsd_prog_1, nconf))
 	{
-	  log_msg("cannot create udp service.");
-	  return 1;
-	}
-      if (!svc_register(main_transp, YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
-			ypxfrd_freebsd_prog_1, IPPROTO_UDP))
-	{
-	  log_msg("unable to register (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS, udp).");
+	  log_msg ("unable to register (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS) for %s, %s.",
+		   nconf->nc_protofmly, nconf->nc_proto);
 	  return 1;
 	}
     }
-
-  if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_STREAM))
-    {
-      if (_rpcfdtype == 0 && my_port >= 0)
-	{
-	  my_socket = socket (AF_INET, SOCK_STREAM, 0);
-	  if (my_socket < 0)
-	    {
-	      log_msg ("%s: can not create TCP ",progname);
-	      return 1;
-	    }
-
-	  memset((char *) &socket_address, 0, sizeof(socket_address));
-	  socket_address.sin_family = AF_INET;
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (my_port);
-
-	  result = bind (my_socket, (struct sockaddr *) &socket_address,
-			 sizeof (socket_address));
-	  if (result < 0)
-	    {
-	      log_msg("%s: can not bind TCP ",progname);
-	      return 1;
-	    }
-	}
-      main_transp = svctcp_create(my_socket, 0, 0);
-      if (main_transp == NULL)
-	{
-	  log_msg("%s: cannot create tcp service\n", progname);
-	  exit(1);
-	}
-      if (!svc_register(main_transp, YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
-			ypxfrd_freebsd_prog_1, IPPROTO_TCP))
-	{
-	  log_msg("%s: unable to register (YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS, tcp)\n",
-		 progname);
-	  exit(1);
-	}
-    }
+  __rpc_endconf (nc_handle);
 
   if (_rpcpmstart)
     {
@@ -456,10 +459,10 @@ main (int argc, char **argv)
       alarm (_RPCSVC_CLOSEDOWN);
     }
 
-  /* If we use systemd as an init system, we may want to give it 
+  /* If we use systemd as an init system, we may want to give it
      a message, that this daemon is ready to accept connections.
-     At this time, sockets for receiving connections are already 
-     created, so we can say we're ready now. It is a nop if we 
+     At this time, sockets for receiving connections are already
+     created, so we can say we're ready now. It is a nop if we
      don't use systemd. */
   announce_ready();
 
